#!/usr/bin/env python
# -*- coding: utf-8 -*-
#=========================================================================
# Automate de prêt - version 0.4
# Copyright (C) 2009, 2010, 2011 Laurent Le Guillou & Thierry Le Guillou
# <llg@lpnhep.in2p3.fr>, <tlg1@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301  USA
#
#=========================================================================

import sys
import os, os.path
import datetime
import gtk, gobject, pango

#=========================================================================

import utils

from database import *

#=========================================================================
# Logging function

def info(msg, level=0, filehandle=sys.stderr):
    date = str(datetime.datetime.utcnow())
    print >>filehandle, ( "[%s]: %s%s" % 
                          (date, level*'  ', msg)) 

#=========================================================================
# States for the automaton

STATE_WELCOME     = 0
STATE_PERSON_LIST = 1
STATE_PERSON      = 2
STATE_SEARCH      = 3
STATE_RETURN      = 4
STATE_RETURNED    = 5
STATE_ABOUT       = 6

#=========================================================================
# Sequence marker for the barcode reader
#
# --------------------------------------------------
# Reader from BUPMC Chimie-Physique Recherche
# Metrologic’s MS9520 Voyager
#
# Barcode 3800000179 :
# 65407 FF7F 77 Num_Lock
# 65459 FFB3 89 KP_3
# 65464 FFB8 80 KP_8
# 65456 FFB0 90 KP_0
# 65456 FFB0 90 KP_0
# 65456 FFB0 90 KP_0
# 65456 FFB0 90 KP_0
# 65456 FFB0 90 KP_0
# 65457 FFB1 87 KP_1
# 65463 FFB7 79 KP_7
# 65465 FFB9 81 KP_9
# 65293 FF0D 36 Return
# 65407 FF7F 77 Num_Lock
#
# BARCODE_START = gtk.keysyms.Num_Lock
BARCODE_START = gtk.keysyms.Tab
BARCODE_STOP  = gtk.keysyms.Return
#
BARCODE_KEYS = [gtk.keysyms._0, gtk.keysyms.KP_0,
                gtk.keysyms._1, gtk.keysyms.KP_1,
                gtk.keysyms._2, gtk.keysyms.KP_2,
                gtk.keysyms._3, gtk.keysyms.KP_3,
                gtk.keysyms._4, gtk.keysyms.KP_4,
                gtk.keysyms._5, gtk.keysyms.KP_5,
                gtk.keysyms._6, gtk.keysyms.KP_6,
                gtk.keysyms._7, gtk.keysyms.KP_7,
                gtk.keysyms._8, gtk.keysyms.KP_8,
                gtk.keysyms._9, gtk.keysyms.KP_9]
#
BARCODE_TRANSLATION = { gtk.keysyms._0 : '0', gtk.keysyms.KP_0 : '0',
                        gtk.keysyms._1 : '1', gtk.keysyms.KP_1 : '1',
                        gtk.keysyms._2 : '2', gtk.keysyms.KP_2 : '2',
                        gtk.keysyms._3 : '3', gtk.keysyms.KP_3 : '3',
                        gtk.keysyms._4 : '4', gtk.keysyms.KP_4 : '4',
                        gtk.keysyms._5 : '5', gtk.keysyms.KP_5 : '5',
                        gtk.keysyms._6 : '6', gtk.keysyms.KP_6 : '6',
                        gtk.keysyms._7 : '7', gtk.keysyms.KP_7 : '7',
                        gtk.keysyms._8 : '8', gtk.keysyms.KP_8 : '8',
                        gtk.keysyms._9 : '9', gtk.keysyms.KP_9 : '9' }
#
# --------------------------------------------------


#=========================================================================
# Path for images/icons

curdir = os.path.dirname(__file__)
iconsdir = os.path.join(curdir, "icons")

pixbuf_person = gtk.gdk.pixbuf_new_from_file_at_size(
    os.path.join(iconsdir, "Person.svg"), 300, 300)
pixbuf_person_unknown = gtk.gdk.pixbuf_new_from_file_at_size(
    os.path.join(iconsdir, "Person.svg"), 300, 300)
pixbuf_person_small = gtk.gdk.pixbuf_new_from_file_at_size(
    os.path.join(iconsdir, "Person.svg"), 100, 100)
# pixbuf_book = gtk.gdk.pixbuf_new_from_file_at_size(
#     os.path.join(iconsdir, "Books.svg"), 150, 150)#accessories-dictionary.svg"
pixbuf_books = gtk.gdk.pixbuf_new_from_file_at_size(
    os.path.join(iconsdir, "Books.svg"), 150, 150)#accessories-dictionary.svg"
pixbuf_books_on_shelf = gtk.gdk.pixbuf_new_from_file_at_size(
    os.path.join(iconsdir, "Books+on+Shelf.svg"), 150, 150)
pixbuf_book_borrowed = gtk.gdk.pixbuf_new_from_file_at_size(
    os.path.join(iconsdir, "Person.svg"), 150, 150)

#=========================================================================
# Some style settings
#

# FONT_SIZE = 10
# FONT_SIZE = 14 # Bien sur laptop pour tests
FONT_SIZE = 15 # Bien
FONT_DESC = "sans %d" % FONT_SIZE
TITLE_FONT_DESC = "sans bold %s" % (2*FONT_SIZE)
info("Font spec: %s" % TITLE_FONT_DESC, level = 1)
SUBTITLE_FONT_DESC = "sans bold %s" % FONT_SIZE
PANGO_FONT_DESC = pango.FontDescription(FONT_DESC)

WIDGET_PADDING = 5

CELL_RENDERER_TEXT_PROPERTIES = {
    'xalign' : 0.0,    # de 0.0 a 1.0 (0.5 = centre)
    'xpad' : 20,       # espacement horizontal (default = 0)
    'yalign' : 0.5,    # de 0.0 a 1.0 (0.5 = centre)
    'ypad' : 10,
    #"font-desc" : PANGO_FONT_DESC,
    'ellipsize' : pango.ELLIPSIZE_END,
    'ellipsize-set' : True
    }

CELL_RENDERER_PIXBUF_PROPERTIES = {
    'xalign' : 0.5,    # de 0.0 a 1.0 (0.5 = centre)
    'xpad' : 20,       # espacement horizontal (default = 0)
    'yalign' : 0.5,    # de 0.0 a 1.0 (0.5 = centre)
    'ypad' : 0,
    }

button_bgcolor = gtk.gdk.rgb_get_colormap().alloc_color('#89D960')

#==== Welcome screen =====================================================

class WelcomeScreen(gtk.VBox): # Screen to welcome the user
    
    def __init__(self):
        gtk.VBox.__init__(self, False)
        self.show()

        # ---------------------------------------------------

        box_title = gtk.VBox()
        box_title.show()

        label_subtitle = gtk.Label("")
        label_subtitle.set_alignment(0.5, 0.5)
        label_subtitle.set_markup("<span foreground='#CC0000' font-desc='%s'>Bibliothèque du LPNHE</span>\n" % SUBTITLE_FONT_DESC)
        label_subtitle.show()
        box_title.pack_start(label_subtitle)

        label_title = gtk.Label("")
        label_title.set_alignment(0.5, 0.5)
        label_title.set_markup("<span foreground='#CC0000' font-desc='%s'><b>Emprunts d'Ouvrages / <i>Book Borrowings</i></b></span>" % TITLE_FONT_DESC)
        label_title.show()
        box_title.pack_start(label_title)

        # ---------------------------------------------------

        label_borrow = gtk.Label("")
        label_borrow.set_alignment(0, 0.5)
        # label_borrow.set_markup("<span foreground='#000000' font-desc='%s'><b>Emprunter un ouvrage</b>\nPrésentez votre carte devant le lecteur de codes-barres,\npuis présentez les ouvrages que vous empruntez.\n\nSi vous avez oublié votre carte, cliquez sur le bouton 'J'ai oublié ma Carte'.\n\n<b><i>Borrow a book</i></b>\n<i>Present your card in front of the barcode reader,\nthen present the books you are borrowing.\n\nIf you forgot your card, click on 'I forgot my Card' below.</i></span>\n" % FONT_DESC)
        label_borrow.set_markup("<span foreground='#000000' font-desc='%s'><b>Emprunter un Ouvrage</b>\nCliquez sur 'Emprunter un Ouvrage', identifiez vous en cliquant sur votre nom,\npuis présentez les ouvrages que vous empruntez.\n\n<b><i>Borrow a Book</i></b>\n<i>Click on 'Borrow a Book', identify yourself by selecting your name,\nthen present the books you are borrowing.</i></span>\n" % FONT_DESC)
        label_borrow.show()

        # ---------------------------------------------------

        label_return = gtk.Label("")
        label_return.set_alignment(0, 0.5)
        label_return.set_markup("<span foreground='#000000' font-desc='%s'><b>Rendre un ouvrage</b>\nPrésentez l'ouvrage devant le lecteur de codes-barres,\npuis déposez le sur la table.\n\n<b><i>Return a book</i></b>\n<i>Present the book you return in front of the barcode reader,\nthen leave it on the table.</i></span>\n" % FONT_DESC)
        label_return.show()

        # ---------------------------------------------------

        image_loop = gtk.Image()
        image_loop.set_from_file(os.path.join(iconsdir, "animation-main.gif"))
        image_loop.show()

        # ---------------------------------------------------

        # table = gtk.Table(4, 3, True)
        table = gtk.Table(3, 3, True)
        # table = gtk.Table(4, 3, False)
        # table_search.set_border_width(8)
        table.set_row_spacings(0)#5)
        table.set_col_spacings(WIDGET_PADDING)

        table.attach(
            box_title, 0, 3, 0, 1, xoptions=gtk.FILL|gtk.EXPAND,
            # yoptions=gtk.FILL|gtk.EXPAND, 
            yoptions=0, 
            xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            label_borrow, 1, 3, 1, 2, xoptions=gtk.FILL,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            label_return, 1, 3, 2, 3, xoptions=gtk.FILL,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            image_loop, 0, 1, 1, 3, xoptions=gtk.FILL,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.show()

        #box_person_id.set_property("visible", False)
        #scrolledwin_books.set_property("visible", False)

        self.pack_start(table, True, True)


#==== Person List screen =================================================

class PersonListScreen(gtk.VBox): 
    # Class for the list of persons screen
    
    #---- Creating the UI ------------------------------------------------------

    def __init__(self):
        gtk.VBox.__init__(self, False)
        self.show()

        #---- A-Z dictionary for A-Z shortcut ----------------------------------

        self.a_z_indices = {}
        for i in xrange(ord('A'),ord('Z')+1):
            self.a_z_indices[chr(i)] = 0

        #---- Box for message to user ------------------------------------------

        box_messages = gtk.VBox()

        # self.icon_messages = gtk.Image()
        # self.icon_messages.set_from_file(os.path.join(iconsdir, "borrowing.gif"));
        # self.icon_messages.show()

        self.title_label = gtk.Label("")
        self.title_label.set_markup("<span foreground='#CC0000' font-desc='%s'><b>Cliquez sur votre Nom / <i>Click on your Name</i></b></span>" % TITLE_FONT_DESC)
        self.title_label.show()

        # self.label_messages = gtk.Label("")
        # self.label_messages.set_markup("<span foreground='#000000' font-desc='%s'>\nSélectionner votre nom dans la liste :\n<i>Click on your name in the list :</i></span>" % FONT_DESC)
        # self.label_messages.show()

        # box_messages.pack_start(self.icon_messages, False, False)
        box_messages.pack_start(self.title_label, True, True)
        # box_messages.pack_start(self.label_messages, True, True)
        box_messages.show()

        #---- Box for user info (name, office, phone) --------------------------

        person_image = gtk.Image()
        person_image.set_from_pixbuf(pixbuf_person_unknown)
        person_image.show()

        label_message = gtk.Label("")
        label_message.set_markup("<span foreground='#000000' font-desc='%s'>\nSélectionner votre nom dans la liste :\n<i>Select your name in the list :</i></span>" % FONT_DESC)
        label_message.show()

        box_person_id = gtk.VBox(False)
        box_person_id.pack_start(person_image, False, False)
        box_person_id.pack_start(label_message, False, False)
        box_person_id.show()

        #---- Box for persons (all of them) ------------------------------------

        self.a_z_toolbar = gtk.Toolbar()
        self.a_z_toolbar.set_properties(
            **{"orientation":gtk.ORIENTATION_HORIZONTAL,
               "show-arrow": False, 
               "toolbar-style":gtk.TOOLBAR_TEXT})

        self.a_z_toolbar.show()

 	treemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT)

        self.tree_person_list = gtk.TreeView(treemodel)
        self.tree_person_list.set_property("can-focus", True)
        self.tree_person_list.set_rules_hint(True)
        self.tree_person_list.set_headers_visible(False)
                
        self.treeselection = self.tree_person_list.get_selection()
        # treeselection.set_mode(gtk.SELECTION_NONE)
        self.treeselection.set_mode(gtk.SELECTION_SINGLE)
        
 	renderer = gtk.CellRendererText()
        renderer.set_properties(**CELL_RENDERER_TEXT_PROPERTIES)
        
	rendererpixbuf = gtk.CellRendererPixbuf()
        rendererpixbuf.set_properties(**CELL_RENDERER_PIXBUF_PROPERTIES)
        
        self.tree_person_list.append_column(
            gtk.TreeViewColumn("Icon", rendererpixbuf, pixbuf=0))
 	self.tree_person_list.append_column(
            gtk.TreeViewColumn("Name", renderer, markup=1))
        self.tree_person_list.show()

        scrolledwin_persons = gtk.ScrolledWindow()
        scrolledwin_persons.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwin_persons.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        #scrolledwin_persons.set_border_width(8)
        scrolledwin_persons.add(self.tree_person_list)
	scrolledwin_persons.show()

        scrolledwin_persons_box = gtk.VBox(False)
        scrolledwin_persons_box.pack_start(self.a_z_toolbar, False, False)
        scrolledwin_persons_box.pack_start(scrolledwin_persons, True, True)
        scrolledwin_persons_box.show()

        # box_person.pack_start(scrolledwin_persons, True, True, 0)

        table = gtk.Table(4, 3, True)
        table.set_row_spacings(0) #5)
        table.set_col_spacings(5)

        table.attach(
            box_messages, 0, 3, 0, 1, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            box_person_id, 0, 1, 1, 4, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            scrolledwin_persons_box, 1, 3, 1, 4, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=gtk.FILL, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.show()

        #box_person_id.set_property("visible", False)
        #scrolledwin_persons.set_property("visible", False)

        self.pack_start(table, True, True)

        self.display_all_persons()

        a_z_keys = self.a_z_indices.keys()
        a_z_keys.sort()
        for letter in a_z_keys:
            # deprecated en 2.4 
            # toolbar.append_item(text, tooltip_text, tooltip_private_text, icon, callback, user_data=None)
            # sinon 
            toolbutton = gtk.ToolButton(icon_widget=None, label=letter)
            toolbutton.connect("clicked", self.scrolltoperson, self.a_z_indices[letter])
            toolbutton.show()
            self.a_z_toolbar.insert(toolbutton, -1)


    def scrolltoperson(self, toolbutton, index):
        self.tree_person_list.scroll_to_cell(index, use_align = True, row_align = 0.0)

    #---- Display all people ---------------------------------------------------

    def display_all_persons(self):
        treemodel = self.tree_person_list.get_model()
        treemodel.clear()
        
        self.a_z_indices = {}
        index = 0

        for person in Person.select(Person.q.name != 'NOT ON SHELF', 
                                    orderBy = Person.q.name):

            initial = (person.name.strip())[0].upper()
            if not(self.a_z_indices.has_key(initial)):
                self.a_z_indices[initial] = index

            treemodel.append(
                (pixbuf_person_small,
                 ("<span font-desc='%s'><b>%s</b>\nBureau / <i>Office</i> : %s \nTéléphone : %s</span>" % 
                  (FONT_DESC, 
                   utils.markup_protect(person.name), 
                   utils.markup_protect(person.office), 
                   utils.markup_protect(person.phone) 
                   )
                  ),
                 person ))

            index += 1

                                      
#==== Person screen ======================================================

class PersonScreen(gtk.VBox): # Class to represent the Person/Borrowing screen
    
    #---- Creating the UI ------------------------------------------------------

    def __init__(self):
        gtk.VBox.__init__(self, False)
        self.show()

        #---- Box for message to user ------------------------------------------

        box_messages = gtk.VBox()

        # self.icon_messages = gtk.Image()
        # self.icon_messages.set_from_file(os.path.join(iconsdir, "borrowing.gif"));
        # self.icon_messages.show()

        self.title_label = gtk.Label("")
        self.title_label.set_markup("<span foreground='#CC0000' font-desc='%s'><b>Emprunts d'Ouvrages / <i>Book Borrowings</i></b></span>" % TITLE_FONT_DESC)
        self.title_label.show()

        self.label_messages = gtk.Label("")
        self.label_messages.set_markup("<span foreground='#000000' font-desc='%s'>\nPour emprunter ou rendre un ouvrage, présentez son code barre devant le lecteur\n<i>To borrow or return a book, present its barcode in front of the barcode reader</i></span>" % FONT_DESC)
        self.label_messages.show()

        # box_messages.pack_start(self.icon_messages, False, False)
        box_messages.pack_start(self.title_label, True, True)
        box_messages.pack_start(self.label_messages, True, True)
        box_messages.show()

        #---- Box for user info (name, office, phone) --------------------------

        person_image = gtk.Image()
        person_image.set_from_pixbuf(pixbuf_person)
        person_image.show()

        self.label_name = gtk.Label("")
        self.label_name.show()
        self.label_office = gtk.Label("")
        self.label_office.show()
        self.label_phone = gtk.Label("")
        self.label_phone.show()
        self.label_books = gtk.Label("")
        self.label_books.show()

        box_person_id = gtk.VBox(False)
        box_person_id.pack_start(person_image, False, False)
        box_person_id.pack_start(self.label_name, False, False)
        box_person_id.pack_start(self.label_office, False, False)
        box_person_id.pack_start(self.label_phone, False, False)
        box_person_id.pack_start(self.label_books, False, False)
        box_person_id.show()

        #---- Box for user borrowed books --------------------------------------

 	treemodel = gtk.ListStore(gtk.gdk.Pixbuf, str)

        self.tree_book_list = gtk.TreeView(treemodel)
        self.tree_book_list.set_property("can-focus", False)
        self.tree_book_list.set_rules_hint(True)
        self.tree_book_list.set_headers_visible(False)
                
        treeselection = self.tree_book_list.get_selection()
        treeselection.set_mode(gtk.SELECTION_NONE)
        
 	renderer = gtk.CellRendererText()
        renderer.set_properties(**CELL_RENDERER_TEXT_PROPERTIES)
        
	rendererpixbuf = gtk.CellRendererPixbuf()
        rendererpixbuf.set_properties(**CELL_RENDERER_PIXBUF_PROPERTIES)
        
        self.tree_book_list.append_column(
            gtk.TreeViewColumn("Icon",rendererpixbuf,pixbuf=0))
 	self.tree_book_list.append_column(
            gtk.TreeViewColumn("Book", renderer, markup=1))
        self.tree_book_list.show()

        scrolledwin_books = gtk.ScrolledWindow()
        scrolledwin_books.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwin_books.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        #scrolledwin_books.set_border_width(8)
        scrolledwin_books.add(self.tree_book_list)
	scrolledwin_books.show()

        # box_person.pack_start(scrolledwin_books, True, True, 0)

        table = gtk.Table(4, 3, True)
        # table_search.set_border_width(8)
        table.set_row_spacings(0)#5)
        table.set_col_spacings(5)

        table.attach(
            box_messages, 0, 3, 0, 1, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            box_person_id, 0, 1, 1, 4, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            scrolledwin_books, 1, 3, 1, 4, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=gtk.FILL, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.show()

        #box_person_id.set_property("visible", False)
        #scrolledwin_books.set_property("visible", False)

        self.pack_start(table, True, True)

    #---- Display books --------------------------------------------------------

    def display_person(self, person=None):
        self.display_person_info(person)
        self.display_books(person.books)

    def display_books(self, books=[]):
        treemodel = self.tree_book_list.get_model()
        treemodel.clear()
        
        for book in books:
            treemodel.append(
                (pixbuf_books,
                 ("<span font-desc='%s'><b>%s</b>\n%s \n%s / %s \n%s</span>" % 
                  (FONT_DESC, 
                   utils.markup_protect(book.title), 
                   utils.markup_protect(book.authors), 
                   utils.markup_protect(book.publisher), 
                   book.pubyear, book.location))))

    def clear_books(self):
        self.display_books([])

    def display_person_info(self, person=None):
        '''
        Update the Person information (name, office, phone) displayed on this window.
        '''

        txt_name = ("<span foreground=\"orange\" font-desc='%s'><b>Personne n'est connecté</b></span>" % FONT_DESC)
        txt_office = ""
        txt_phone = ""
        txt_books = ""
        if person is not None:
            txt_name = "<span font-desc='%s'><b>%s</b></span>"% (FONT_DESC, 
                                                                 str(person.name))
            txt_office = "<span font-desc='%s'>Bureau / <i>Office</i> : %s</span>" % (FONT_DESC, 
                                                             str(person.office))
            txt_phone = "<span font-desc='%s'>Tel : %s</span>" % (FONT_DESC, 
                                                            str(person.phone))
            nbooks = len(list(person.books))
            if nbooks == 0:
                txt_books = ("<span foreground='#CC0000' font-desc='%s'>Aucun emprunt\n<i>No borrowed book</i></span>" % FONT_DESC)
            else:
                terminal_s = ''
                if nbooks > 1:
                    terminal_s = 's'
                txt_books = "<span foreground='#CC0000' font-desc='%s'>%d ouvrage%s emprunté%s\n%d <i>borrowed book%s</i></span>" % (FONT_DESC, nbooks, terminal_s, terminal_s, nbooks, terminal_s)
            
        self.label_name.set_markup(txt_name)
        self.label_office.set_markup(txt_office)
        self.label_phone.set_markup(txt_phone)
        self.label_books.set_markup(txt_books)


#==== Search screen ======================================================

class SearchScreen(gtk.VBox):
    def __init__(self):
        gtk.VBox.__init__(self)
        self.show()

        label_title = gtk.Label("")
        label_title.set_markup('<span font_desc="%s">Titre / <i>Title</i> :</span>'
                               % FONT_DESC)
        label_title.set_alignment(0, 0.5)
        label_title.show()
        
        label_authors = gtk.Label("")
        label_authors.set_markup('<span font_desc="%s">Auteur / <i>Author</i> :</span>'
                                 % FONT_DESC)
        label_authors.set_alignment(0, 0.5)
        label_authors.show()

        label_publisher = gtk.Label("")
        label_publisher.set_markup('<span font_desc="%s">Editeur / <i>Publisher</i> :</span>'
                                   % FONT_DESC)
        label_publisher.set_alignment(0, 0.5)
        label_publisher.show()

        label_pubyear = gtk.Label("")
        label_pubyear.set_markup('<span font_desc="%s">Année / <i>Year</i></span> :' 
                                 % FONT_DESC)
        label_pubyear.set_alignment(0, 0.5)
        label_pubyear.show()

        self.entry_title = gtk.Entry()
        self.entry_title.modify_font(PANGO_FONT_DESC)
        self.entry_title.show()
        
        self.entry_authors = gtk.Entry()
        self.entry_authors.modify_font(PANGO_FONT_DESC)
        self.entry_authors.show()
        
        self.entry_publisher = gtk.Entry()
        self.entry_publisher.modify_font(PANGO_FONT_DESC)
        self.entry_publisher.show()
        
        self.entry_pubyear = gtk.Entry()
        self.entry_pubyear.modify_font(PANGO_FONT_DESC)
        self.entry_pubyear.show()

        vsep = gtk.VSeparator()
        vsep.show()

        box_A = gtk.HBox(False)
        box_A.pack_start(self.entry_title, True, True, 0)
        box_A.pack_start(label_pubyear, False, False, 15)
        box_A.pack_start(self.entry_pubyear, False, False, 0)
        box_A.show()

        self.button_search = gtk.Button(" Rechercher / Search ")
        label = self.button_search.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.button_search.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            # box_messages.modify_bg(gtk.STATE_INSENSITIVE, black)
            # box_messages.modify_base(i, button_bgcolor)
            self.button_search.modify_bg(i, button_bgcolor)

        self.button_clear = gtk.Button("Effacer / Clear")
        label = self.button_clear.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.button_clear.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            # box_messages.modify_bg(gtk.STATE_INSENSITIVE, black)
            # box_messages.modify_base(i, button_bgcolor)
            self.button_clear.modify_bg(i, button_bgcolor)

        table_search = gtk.Table(2,2, False)
        #table_search.set_border_width(8)
        table_search.set_row_spacings(0)#5)
        table_search.set_col_spacings(5)
        table_search.show()

        table_search.attach(
            label_title, 0, 1, 0, 1, xoptions=gtk.FILL,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table_search.show()

        table_search.attach(
            box_A, 1, 4, 0, 1, xoptions=gtk.FILL,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table_search.show()
        table_search.attach(
            label_authors, 0, 1, 1, 2, xoptions=gtk.FILL,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table_search.attach(
            self.entry_authors, 1, 2, 1, 2, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table_search.attach(
            label_publisher, 2, 3, 1, 2, xoptions=gtk.FILL,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table_search.attach(
            self.entry_publisher, 3, 4, 1, 2, xoptions=gtk.FILL|gtk.EXPAND,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table_search.attach(
            self.button_search, 4, 5, 0, 1, xoptions=gtk.FILL,
            yoptions=0, xpadding=0, ypadding=WIDGET_PADDING)
        table_search.attach(
            self.button_clear, 4, 5, 1, 2, xoptions=gtk.FILL,
            yoptions=0, xpadding=0, ypadding=WIDGET_PADDING)

        box_search = gtk.HBox()
        box_search.show()
        self.pack_start(box_search, False, True, padding=20)
        box_search.pack_start(table_search, True, True, 0)

        #self.pack_start(table_search, False, True, 15)

 	treemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, gtk.gdk.Pixbuf, str)

        self.tree_book_list = gtk.TreeView(treemodel)
        self.tree_book_list.set_rules_hint(True)
        self.tree_book_list.set_headers_visible(False)
        # self.tree_book_list.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_VERTICAL)
                
        treeselection = self.tree_book_list.get_selection()
        treeselection.set_mode(gtk.SELECTION_NONE)
        
 	renderer = gtk.CellRendererText()
        renderer.set_properties(**CELL_RENDERER_TEXT_PROPERTIES)

	rendererpixbuf = gtk.CellRendererPixbuf()
        rendererpixbuf.set_properties(**CELL_RENDERER_PIXBUF_PROPERTIES)

        colA = gtk.TreeViewColumn("First Name",rendererpixbuf,pixbuf=0)
        # self.tree_book_list.append_column(colA)

        colB = gtk.TreeViewColumn("Last Name", renderer, markup=1)
        colB.set_expand(True)
        colB.set_min_width(50)
 	self.tree_book_list.append_column(colB)


        colC = gtk.TreeViewColumn("First Name",rendererpixbuf,pixbuf=2)
        self.tree_book_list.append_column(colC)

        colD = gtk.TreeViewColumn("Last Name", renderer, markup=3)
        # colD.set_expand(True)
        colD.set_expand(False)
        colD.set_min_width(300)
        self.tree_book_list.append_column(colD)
        
        self.tree_book_list.show()

        scrolledwin_books = gtk.ScrolledWindow()
        scrolledwin_books.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwin_books.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        #scrolledwin_books.set_border_width(8)
        scrolledwin_books.add(self.tree_book_list)
	scrolledwin_books.show()
        self.pack_start(scrolledwin_books, True, True, 0)
        

    def clear(self, widget):
        for i in [
            self.entry_title, self.entry_authors,
            self.entry_publisher, self.entry_pubyear]:
            i.set_text("")
        

    def search(self, widget):

        # Get the search patterns from the text entries

        title_pattern     = self.entry_title.get_text()
        authors_pattern   = self.entry_authors.get_text()
        publisher_pattern = self.entry_publisher.get_text()
        pubyear_pattern   = self.entry_pubyear.get_text()

        results = Book.selection(title_pattern, authors_pattern, 
                                 publisher_pattern, pubyear_pattern)

        self.display_results(results)
        

    def display_results(self, results=[]):
        treemodel = self.tree_book_list.get_model()
        treemodel.clear()
        
        for book in results:
            pix = pixbuf_books_on_shelf
            txt = ( "<span font-desc='%s'><b>En rayon / <i>On shelf</i>\n\n%s</b></span>" % 
                    (FONT_DESC, book.location) )

            if book.person is not None:
                pix = pixbuf_book_borrowed
                txt = ( ("<span font-desc='%s'><b>Emprunté / <i>Borrowed</i></b>\n" + 
                         "par / <i>by</i>\n<b>%s</b>\n%s / Tél : %s</span>" )
                        % ( FONT_DESC, 
                            book.person.name, 
                            book.person.office, 
                            book.person.phone ) )
                
            treemodel.append(
                ( pixbuf_books, 
                  ("<span font-desc='%s'><b>%s</b>\n%s \n%s / %s</span>" % 
                   ( FONT_DESC, 
                     utils.markup_protect(book.title), 
                     utils.markup_protect(book.authors), 
                     utils.markup_protect(book.publisher), 
                     book.pubyear)),
                  pix, 
                  txt ))

#==== Return screen ======================================================

class ReturnScreen(gtk.VBox): # Screen for anonymous book returns
    
    def __init__(self):
        gtk.VBox.__init__(self, False)
        self.show()

        # ---------------------------------------------------
        
        box_title = gtk.VBox()
        box_title.show()
        
        label_subtitle = gtk.Label("")
        label_subtitle.set_alignment(0.5, 0.5)
        label_subtitle.set_markup("<span foreground='#CC0000' font-desc='%s'>Bibliothèque du LPNHE</span>\n" % SUBTITLE_FONT_DESC)
        label_subtitle.show()
        box_title.pack_start(label_subtitle)
        
        label_title = gtk.Label("")
        label_title.set_alignment(0.5, 0.5)
        label_title.set_markup("<span foreground='#CC0000' font-desc='%s'><b>Retour d'Ouvrages / <i>Book Return</i></b></span>" % TITLE_FONT_DESC)
        label_title.show()
        box_title.pack_start(label_title)
        
        # ---------------------------------------------------

        box_return = gtk.VBox()
        
        # ---
        
 	treemodel = gtk.ListStore(gtk.gdk.Pixbuf, str)
        
        self.tree_book_list = gtk.TreeView(treemodel)
        self.tree_book_list.set_property("can-focus", False)
        self.tree_book_list.set_rules_hint(True)
        self.tree_book_list.set_headers_visible(False)
        
        treeselection = self.tree_book_list.get_selection()
        treeselection.set_mode(gtk.SELECTION_NONE)
        
 	renderer = gtk.CellRendererText()
        renderer.set_properties(**CELL_RENDERER_TEXT_PROPERTIES)

	rendererpixbuf = gtk.CellRendererPixbuf()
        rendererpixbuf.set_properties(**CELL_RENDERER_PIXBUF_PROPERTIES)
        
        self.tree_book_list.append_column(
            gtk.TreeViewColumn("First Name",rendererpixbuf,pixbuf=0))
 	self.tree_book_list.append_column(
            gtk.TreeViewColumn("Last Name", renderer, markup=1))
        self.tree_book_list.show()
        
        scrolledwin_books = gtk.ScrolledWindow()
        scrolledwin_books.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwin_books.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        #scrolledwin_books.set_border_width(8)
        scrolledwin_books.add(self.tree_book_list)
	scrolledwin_books.show()
        
        # box_return.pack_start(label_return)
        box_return.pack_start(scrolledwin_books, True, True)
        
        box_return.show()

        # ---------------------------------------------------

        label_message = gtk.Label("")
        label_message.set_alignment(0, 0.5)
        label_message.set_markup("<span foreground='#000000' font-desc='%s'>Merci de déposer vos ouvrages sur la table.\n<i>Please put your returned books on the table.</i></span>\n" % FONT_DESC)
        label_message.show()

        # ---------------------------------------------------

        image_loop = gtk.Image()
        image_loop.set_from_file(os.path.join(iconsdir, "animation-return.gif"))
        image_loop.show()

        # ---------------------------------------------------

        table = gtk.Table(3, 3, True)
        # table = gtk.Table(4, 3, False)
        # table_search.set_border_width(8)
        table.set_row_spacings(0)#5)
        table.set_col_spacings(WIDGET_PADDING)
        
        table.attach(
            box_title, 0, 3, 0, 1, xoptions=gtk.FILL|gtk.EXPAND,
            # yoptions=gtk.FILL|gtk.EXPAND, 
            yoptions=0, 
            xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            box_return, 1, 3, 1, 2, xoptions=gtk.FILL,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            label_message, 1, 3, 2, 3, xoptions=gtk.FILL,
            yoptions=gtk.FILL, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            image_loop, 0, 1, 1, 3, xoptions=gtk.FILL,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.show()

        self.pack_start(table, True, True)


    def display_return(self, book):
        treemodel = self.tree_book_list.get_model()
        treemodel.clear()
        
        treemodel.append(
            (pixbuf_books,
             ("<span font-desc='%s'><b>%s</b>\n%s \n%s / %s \n%s</span>" % 
              (FONT_DESC, 
               utils.markup_protect(book.title), 
               utils.markup_protect(book.authors), 
               utils.markup_protect(book.publisher), 
               book.pubyear, book.location))))
        

#==== Returned screen (book has already been returned)====================
# user most propably forgot to present its card first

class ReturnedScreen(gtk.VBox): 
    
    def __init__(self):
        gtk.VBox.__init__(self, False)
        self.show()

        # ---------------------------------------------------

        box_title = gtk.VBox()
        box_title.show()

        label_subtitle = gtk.Label("")
        label_subtitle.set_alignment(0.5, 0.5)
        label_subtitle.set_markup("<span foreground='#CC0000' font-desc='%s'>Bibliothèque du LPNHE</span>\n" % SUBTITLE_FONT_DESC)
        label_subtitle.show()
        box_title.pack_start(label_subtitle)

        label_title = gtk.Label("")
        label_title.set_alignment(0.5, 0.5)
        label_title.set_markup("<span foreground='#CC0000' font-desc='%s'><b>Identifez vous d'abord / <i>Identify yourself first</i></b></span>" % TITLE_FONT_DESC)
        label_title.show()
        box_title.pack_start(label_title)

        # ---------------------------------------------------

        box_message = gtk.VBox()

        image_loop = gtk.Image()
        image_loop.set_from_file(os.path.join(iconsdir, "animation-card.gif"))
        image_loop.set_alignment(0.5, 0.5)
        image_loop.show()

        label_message = gtk.Label("")
        label_message.set_alignment(0.5, 0.5)
        label_message.set_markup("<span foreground='#000000' font-desc='%s'>Pour emprunter un ouvrage, identifiez vous, puis présentez l'ouvrage.\n\n<i>To borrow a book, please identify yourself, then present the book.</i></span>\n" % FONT_DESC)
        label_message.show()

        box_message.pack_start(image_loop, True, True)
        box_message.pack_start(label_message, True, True)
        box_message.show()

        # ---------------------------------------------------

        table = gtk.Table(3, 3, True)
        # table = gtk.Table(4, 3, False)
        # table_search.set_border_width(8)
        table.set_row_spacings(0)#5)
        table.set_col_spacings(WIDGET_PADDING)

        table.attach(
            box_title, 0, 3, 0, 1, xoptions=gtk.FILL|gtk.EXPAND,
            # yoptions=gtk.FILL|gtk.EXPAND, 
            yoptions=0, 
            xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            box_message, 0, 3, 1, 3, 
            xoptions=gtk.FILL, yoptions=gtk.FILL, 
            xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        # table.attach(
        #     label_message, 0, 3, 2, 3, xoptions=gtk.FILL,
        #     yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        # table.attach(
        #     image_loop, 1, 2, 1, 2, xoptions=gtk.FILL,
        #     yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.show()

        self.pack_start(table, True, True)

#==== About screen =======================================================

class AboutScreen(gtk.VBox): # A propos...
    
    def __init__(self):
        gtk.VBox.__init__(self, False)
        self.show()

        # ---------------------------------------------------

        box_title = gtk.VBox()
        box_title.show()

        label_subtitle = gtk.Label("")
        label_subtitle.set_alignment(0.5, 0.5)
        label_subtitle.set_markup("<span foreground='#CC0000' font-desc='%s'>Bibliothèque du LPNHE</span>\n" % SUBTITLE_FONT_DESC)
        label_subtitle.show()
        box_title.pack_start(label_subtitle)

        label_title = gtk.Label("")
        label_title.set_alignment(0.5, 0.5)
        label_title.set_markup("<span foreground='#CC0000' font-desc='%s'><b>Automate de prêt</b></span>" % TITLE_FONT_DESC)
        label_title.show()
        box_title.pack_start(label_title)

        # ---------------------------------------------------

        label_copyright = gtk.Label("")
        label_copyright.set_alignment(0, 0.5)
        label_copyright.set_markup("<span foreground='#000000' font-desc='%s'><b>Automate de prêt</b> - version 0.4\nCopyright (C) 2009, 2010, 2011 Laurent Le Guillou &amp; Thierry Le Guillou\nllg@lpnhe.in2p3.fr - tlg1@free.fr\n\n<b>This program is free software</b>; you can redistribute it and modify\nit under the terms of the <b>GNU General Public License</b> as published by\nthe Free Software Foundation.\n\n\nDéveloppé sous Linux, en Python, les 18-19-20 décembre 2009\n\nMerci aux auteurs de python-gtk, reportlab et SQLobject</span>\n" % FONT_DESC)
        label_copyright.show()

        # ---------------------------------------------------

        # label_return = gtk.Label("")
        # label_return.set_alignment(0, 0)
        # label_return.set_markup("<span foreground='#000000' font-desc='%s'><b>Rendre un ouvrage</b>\nPrésentez l'ouvrage devant le lecteur de codes-barres.\n\n<b><i>Return a book</i></b>\n<i>Present the book you return in front of the barcode reader.</i></span>\n" % FONT_DESC)
        # label_return.show()

        # ---------------------------------------------------

        logo_box = gtk.VBox()

        logo_tux = gtk.Image()
        logo_tux_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
            os.path.join(iconsdir, "Tux-G2.png"), 150, 150)
        logo_tux.set_from_pixbuf(logo_tux_pixbuf)
        logo_tux.show()

        logo_python = gtk.Image()
        logo_python_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
            os.path.join(iconsdir, "python-powered-h.svg"), 150, 150)
        logo_python.set_from_pixbuf(logo_python_pixbuf)
        logo_python.show()

        logo_box.pack_start(logo_tux, True, True, padding=2*WIDGET_PADDING)
        logo_box.pack_start(logo_python, True, True, padding=2*WIDGET_PADDING)
        logo_box.show()

        # ---------------------------------------------------

        table = gtk.Table(3, 3, True)
        # table = gtk.Table(4, 3, False)
        # table_search.set_border_width(8)
        table.set_row_spacings(0)#5)
        table.set_col_spacings(WIDGET_PADDING)

        table.attach(
            box_title, 0, 3, 0, 1, xoptions=gtk.FILL|gtk.EXPAND,
            # yoptions=gtk.FILL|gtk.EXPAND, 
            yoptions=0, 
            xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            label_copyright, 1, 3, 1, 3, xoptions=gtk.FILL,
            yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        # table.attach(
        #     label_return, 1, 3, 2, 3, xoptions=gtk.FILL,
        #     yoptions=gtk.FILL|gtk.EXPAND, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)
        table.attach(
            logo_box, 0, 1, 1, 3, xoptions=0,
            yoptions=0, xpadding=WIDGET_PADDING, ypadding=WIDGET_PADDING)

        table.show()

        #box_person_id.set_property("visible", False)
        #scrolledwin_books.set_property("visible", False)

        self.pack_start(table, True, True)


#==== Button Bar (with input for barcode reader) =========================

class ButtonBar(gtk.HBox):

    def __init__(self):
        gtk.HBox.__init__(self)
        self.show()

        #---- Search Button (to switch to Search Screen) -----------------------

        self.search_button = gtk.Button(" Rechercher un Ouvrage / Search a Book ")
        self.search_button.set_property("can-focus", False)
        label = self.search_button.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.pack_start(self.search_button, False, False, padding=WIDGET_PADDING)
        self.search_button.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            self.search_button.modify_bg(i, button_bgcolor)

        #---- Back Button (to switch back) --------------------------------------

        self.borrowing_button = gtk.Button(" Emprunts / Borrowings ")
        self.borrowing_button.set_property("can-focus", False)
        label = self.borrowing_button.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.pack_start(self.borrowing_button, False, False, padding=WIDGET_PADDING)
        self.borrowing_button.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            self.borrowing_button.modify_bg(i, button_bgcolor)

        #---- Logout Button (to logout) -----------------------------------------

        self.logout_button = gtk.Button(" Quitter / Quit ")
        self.logout_button.set_property("can-focus", False)
        label = self.logout_button.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.pack_start(self.logout_button, False, False, padding=WIDGET_PADDING)
        self.logout_button.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            self.logout_button.modify_bg(i, button_bgcolor)

        #---- Person Search Button (to switch to Person List Screen) ------------

        self.person_search_button = gtk.Button(" Emprunter un Ouvrage / Borrow a Book ")
        self.person_search_button.set_property("can-focus", False)
        label = self.person_search_button.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.pack_start(self.person_search_button, False, False, padding=WIDGET_PADDING)
        self.person_search_button.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            self.person_search_button.modify_bg(i, button_bgcolor)

        #---- About Button (to logout) ------------------------------------------

        self.about_button = gtk.Button(" A propos / About ")
        self.about_button.set_property("can-focus", False)
        label = self.about_button.get_child()
        label.modify_font(PANGO_FONT_DESC)
        self.pack_start(self.about_button, False, False, padding=WIDGET_PADDING)
        self.about_button.show()
        for i in [gtk.STATE_ACTIVE, gtk.STATE_NORMAL, 
                  gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE,
                  gtk.STATE_PRELIGHT]:
            self.about_button.modify_bg(i, button_bgcolor)

        #---- Special Entry for Barcode Reader input ---------------------------

        self.entry_barcode = gtk.Entry()
        self.pack_end(self.entry_barcode, False, False, padding=WIDGET_PADDING)
        self.entry_barcode.set_property("can-focus", False)
        self.entry_barcode.grab_focus()
        self.entry_barcode.show()


    def set_state(self, data = STATE_WELCOME):
        if data == STATE_WELCOME:
            self.search_button.set_property('visible', True)
            self.person_search_button.set_property('visible', True)
            self.borrowing_button.set_property('visible', False)
            self.logout_button.set_property('visible', False)
            self.about_button.set_property('visible', True)
        elif data == STATE_PERSON_LIST:
            self.search_button.set_property('visible', False)
            self.person_search_button.set_property('visible', False)
            self.borrowing_button.set_property('visible', False)
            self.logout_button.set_property('visible', True)
            self.about_button.set_property('visible', False)
        elif data == STATE_PERSON:
            self.search_button.set_property('visible', True)
            self.person_search_button.set_property('visible', False)
            self.borrowing_button.set_property('visible', False)
            self.logout_button.set_property('visible', True)
            self.about_button.set_property('visible', False)
        elif data == STATE_SEARCH:
            self.borrowing_button.set_property('visible', True)
            self.person_search_button.set_property('visible', False)
            self.search_button.set_property('visible', False)
            self.logout_button.set_property('visible', True)
            self.about_button.set_property('visible', False)
        elif data == STATE_RETURN:
            self.borrowing_button.set_property('visible', False)
            self.person_search_button.set_property('visible', False)
            self.search_button.set_property('visible', True)
            self.logout_button.set_property('visible', True)
            self.about_button.set_property('visible', False)
        elif data == STATE_RETURNED:
            self.borrowing_button.set_property('visible', False)
            self.person_search_button.set_property('visible', True)
            self.search_button.set_property('visible', True)
            self.logout_button.set_property('visible', True)
            self.about_button.set_property('visible', False)
        elif data == STATE_ABOUT:
            self.borrowing_button.set_property('visible', False)
            self.person_search_button.set_property('visible', False)
            self.search_button.set_property('visible', True)
            self.logout_button.set_property('visible', True)
            self.about_button.set_property('visible', False)

#==== Main Window of the Application =====================================

class MainWindow(gtk.Window):

    def __init__(self):

        # --- Automaton States -------------------------------------------------
        
        self.automaton_state = STATE_WELCOME
        self.current_person = None
        self.timeout = 5000 # 5000 ms = 5 s
        self.timeout_stack = 0
        self.timeout = gobject.timeout_add(self.timeout, self.timeout_handler)

        self.timeout_state = { # in units of self.timeout
            STATE_WELCOME       : 0,
            STATE_PERSON        : 3,
            STATE_PERSON_LIST   : 4,
            STATE_SEARCH        : 4,
            STATE_RETURN        : 1,
            STATE_RETURNED      : 1,
            STATE_ABOUT         : 2 }

        self.stack = []
        self.barcode_ongoing = False

        # --- Window -----------------------------------------------------------

        gtk.Window.__init__(self)
        self.mainbox = gtk.VBox()
        self.mainbox.show()
        self.add(self.mainbox)

        self.fullscreen()
        # self.unfullscreen()
        self.set_border_width(20)

        self.connect("key-press-event", self.key_press_event)
        self.connect('delete-event', quit)
        self.connect('destroy-event', quit)

        # --- NoteBook (Welcome, Person and Search Screens) ---------------

        self.welcomescreen = WelcomeScreen()
        self.personlistscreen = PersonListScreen()
        self.personscreen = PersonScreen()
        self.searchscreen = SearchScreen()
        self.returnscreen = ReturnScreen()
        self.returnedscreen = ReturnedScreen()
        self.aboutscreen = AboutScreen()

        self.screens = { STATE_WELCOME       : 0,
                         STATE_PERSON_LIST   : 1,
                         STATE_PERSON        : 2,
                         STATE_SEARCH        : 3,
                         STATE_RETURN        : 4,
                         STATE_RETURNED      : 5,
                         STATE_ABOUT         : 6 }

        self.notebook = gtk.Notebook()
        self.notebook.set_show_tabs(False)
        #self.notebook.set_show_tabs(True)
        self.notebook.set_show_border(False)
        self.notebook.append_page(self.welcomescreen)
        self.notebook.append_page(self.personlistscreen)
        self.notebook.append_page(self.personscreen)
        self.notebook.append_page(self.searchscreen)
        self.notebook.append_page(self.returnscreen)
        self.notebook.append_page(self.returnedscreen)
        self.notebook.append_page(self.aboutscreen)
        self.notebook.show()
        self.mainbox.pack_start(self.notebook)

        # --- Search Screen buttons ---------------------------------------

        self.searchscreen.button_search.connect("released", self.searchscreen_search)
        self.searchscreen.button_clear.connect("released", self.searchscreen_clear)

        # --- Search Person buttons ---------------------------------------

        self.personlistscreen.tree_person_list.connect("button-release-event", 
                                                       self.person_select)

        self.personlistscreen.tree_person_list.connect("row-activated", 
                                                       self.person_select)

        # connecter aussi "row-activated"

        # --- Button Bar --------------------------------------------------

        self.buttonbar = ButtonBar()
        self.buttonbar.show()
        self.mainbox.pack_end(self.buttonbar,False, True, 20)

        self.buttonbar.search_button.connect("released", self.goto_search)
        self.buttonbar.person_search_button.connect("released", self.person_search)
        self.buttonbar.borrowing_button.connect("released", self.back_to_borrowings)
        self.buttonbar.logout_button.connect("released", self.back_to_welcome)
        self.buttonbar.about_button.connect("released", self.about)

        # -----------------------------------------------------------------

        self.show()

        # -----------------------------------------------------------------

        self.set_automaton_state(STATE_WELCOME)

        # -----------------------------------------------------------------

    def timeout_handler(self):
        # print "TIMEOUT HANDLER", self.timeout_stack

        if self.automaton_state == STATE_WELCOME:
            # Nothing to do
            self.timeout_stack = 0
            return True
        

        self.timeout_stack += 1

        if self.timeout_stack <= self.timeout_state[self.automaton_state]:
            return True

        # Otherwise it is time to go back to Welcome Screen
        # self.set_automaton_state(STATE_WELCOME)
        #
        self.back_to_welcome(None)
        return True

    def person_search(self, widget):
        self.current_person = None
        self.set_automaton_state(STATE_PERSON_LIST)

    def person_select(self, widget, event, *args):
        # print widget
        # print event
        model, iterator = widget.get_selection().get_selected()
        # print model.get_value(iterator, 1), model.get_value(iterator, 2)
        person = model.get_value(iterator, 2)

        self.current_person = person
        info("Selected person: %s" % str(self.current_person), level = 1)
        # print self.current_person
        self.personscreen.display_person(self.current_person)
        self.set_automaton_state(STATE_PERSON)

    def goto_search(self, widget):
        self.set_automaton_state(STATE_SEARCH)

    def searchscreen_search(self, widget):
        self.timeout_stack = 0
        self.searchscreen.search(None)

    def searchscreen_clear(self, widget):
        self.timeout_stack = 0
        self.searchscreen.clear(None)

    def back_to_borrowings(self, widget):
        # print "BACK", self.current_person, self.automaton_state
        if self.current_person is not None:
            self.set_automaton_state(STATE_PERSON)
        else:
            self.set_automaton_state(STATE_WELCOME)

    def back_to_welcome(self, widget):
        self.current_person = None
        self.set_automaton_state(STATE_WELCOME)
        self.searchscreen.clear(widget = None)


    def about(self, widget):
        self.set_automaton_state(STATE_ABOUT)


    def set_automaton_state(self, data = STATE_WELCOME):
        # print "AUTOMATON: %d -> %d" % (self.automaton_state, data)
        info("Automaton new state: %d -> %d" % 
             (self.automaton_state, data), level = 0)

        self.timeout_stack = 0
        self.buttonbar.entry_barcode.set_text("")            

        self.automaton_state = data
        self.notebook.set_current_page(self.screens[self.automaton_state])
        self.buttonbar.set_state(self.automaton_state)


    def key_press_event(self, widget, event, data=None):
        info("Key pressed: %s %s" % 
             (str(event.keyval), str(event.hardware_keycode)), level = 1)

        self.timeout_stack = 0

        if not(self.barcode_ongoing):
            # No barcode reading
            # normal situation

            if self.automaton_state == STATE_SEARCH:
                if event.keyval in [gtk.keysyms.Return, gtk.keysyms.KP_Enter]:
                    self.searchscreen.search(widget = None)
                    return

            if event.keyval == BARCODE_START:
                self.stack = []
                self.barcode_ongoing = True
                info("Empty key stack", level = 1)
                self.buttonbar.entry_barcode.set_text("")
                return

        else:
            # On going barcode reading

            if event.keyval == BARCODE_START:
                self.stack = []
                self.barcode_ongoing = True
                info("Empty key stack", level = 1)
                self.buttonbar.entry_barcode.set_text("")
                return

            if event.keyval in BARCODE_KEYS:
                self.stack.append(event.keyval)
                self.buttonbar.entry_barcode.insert_text(event.string, 
                                                         position=-1)
                info("Key stack: %s" % str(self.stack), level = 1)
                return

            if event.keyval != BARCODE_STOP:
                # invalid char
                # print "invalid char"
                return

            # Now we know that the char is BARCODE_STOP
            # So it is time to process the barcode and act accordingly

            # First, build the barcode string
            barcode = "".join([BARCODE_TRANSLATION[e] for e in self.stack])
            self.stack = []
            self.barcode_ongoing = False
            self.buttonbar.entry_barcode.set_text("")

            info("Checking barcode: [%s]" % barcode, level = 0)

            if len(barcode) not in [8, 10]:
                # invalid barcode
                return


            if (len(barcode) == 8 and barcode[0:4] == '7585'): 
                # Lecteur (Person) 
                p = Person.find_by_barcode(barcode)
                if p is None:
                    return

                self.current_person = p
                info("Barcode for a person: [%s]" % str(self.current_person), 
                     level = 1)
                # print self.current_person
                self.personscreen.display_person(self.current_person)
                self.set_automaton_state(STATE_PERSON)

            elif (len(barcode) == 10 and barcode[0:3] == '380'): # Livre (Book)
                book = Book.find_by_barcode(barcode)
                # print "book ", barcode
                if book is None:
                    return

                info("Barcode for a book: [%s]" % str(book), level = 1)
                # print book

                if self.automaton_state == STATE_PERSON:

                    if (book.person is None or 
                        book.person != self.current_person):
                        # Borrow the book
                        book.person = self.current_person
                        # message emprunt
                        info("Borrowing book [%s] by person [%s]" % 
                             (str(book), str(self.current_person)), level = 0)
                        # refresh
                        self.personscreen.display_person(self.current_person)
                        self.set_automaton_state(STATE_PERSON)

                    elif book.person == self.current_person:
                        # Return the book
                        book.person = None
                        # message retour
                        info("Returning book [%s] by person [%s]" % 
                             (str(book), str(self.current_person)), level = 0)
                        # print "Retour de ", book, " par ", self.current_person
                        # refresh
                        self.personscreen.display_person(self.current_person)
                        self.set_automaton_state(STATE_PERSON)

                # elif self.automaton_state != STATE_SEARCH:
                else:

                    if book.person is not None:
                        # The book is returned
                        info("Returning book [%s] by person [%s]" % 
                             (str(book), str(book.person)), level = 0)
                        # print "Retour de ", book, " par ", book.person
                        book.person = None
                        # refresh
                        self.returnscreen.display_return(book)
                        self.set_automaton_state(STATE_RETURN)

                    else: # book has already been returned and is on shelf
                        info(("Book [%s] has already been returned." +
                              "To borrow it you should first identify yourself!!!") % 
                             str(book), level = 0)
                        # print ("Book", book, 
                        #        "has already been returned (on shelf)")
                        # print "Please present your card first !!!"
                        self.set_automaton_state(STATE_RETURNED)
                        
                        
#==== Main ===============================================================

def quit(self, *args):
    gtk.main_quit()

def main():

    init() # TODO: gérer proprement l'erreur si pas de DB
    win = MainWindow()
    gtk.main()

if __name__ == '__main__':
    main()
    
#=========================================================================
    
