#! /usr/bin/env python
# -*- Encoding: utf-8 -*- 
#=========================================================================
# Automate de prêt - version 0.2
# Copyright (C) 2009, 2010 Laurent Le Guillou & Thierry Le Guillou
# <llg@lpnhep.in2p3.fr>, <tlg1@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301  USA
#
#=========================================================================

import os, os.path

from sqlobject import *
from sqlobject.sqlbuilder import *

from schema import *

#=========================================================================

# DATABASE_IMAGE="/home/llg/Desktop/LPNHE-Bibliotheque/Automate-Bibliotheque/data/LPNHE-library.sqlite"
DATABASE_IMAGE = os.path.join(os.getenv("HOME"), "Automate-Bibliotheque/data/LPNHE-library.sqlite")

#===============================================================================

def init(debug = False):
    # sqlhub.processConnection = connectionForURI('sqlite:/:memory:')
    sqlhub.processConnection = connectionForURI('sqlite:%s' % DATABASE_IMAGE)
    sqlhub.processConnection.debug = debug

#=========================================================================

def Person_find_by_barcode(barcode=None):
    if barcode == None:
        return None
    l = list(Person.select(Person.q.barcode == barcode))
    if len(l) == 1:
        return l[0]
    return None

Person.find_by_barcode = staticmethod(Person_find_by_barcode)

#=========================================================================

def Book_find_by_barcode(barcode=None):
    if barcode == None:
        return None
    l = list(Book.select(Book.q.barcode == barcode))
    if len(l) == 1:
        return l[0]
    return None

Book.find_by_barcode = staticmethod(Book_find_by_barcode)

#=========================================================================

def Book_selection(title_pattern="", 
                   authors_pattern="", 
                   publisher_pattern="",
                   pubyear_pattern=""):

    criteria = []
    if len(title_pattern) > 0:
        criteria.append(CONTAINSSTRING(Book.q.title, title_pattern))
    if len(authors_pattern) > 0:
        criteria.append(CONTAINSSTRING(Book.q.authors, authors_pattern))
    if len(publisher_pattern) > 0:
        criteria.append(CONTAINSSTRING(Book.q.publisher, publisher_pattern))
    if len(pubyear_pattern) > 0:
        criteria.append(CONTAINSSTRING(Book.q.pubyear, pubyear_pattern))

    if len(criteria) < 1:
        return []

    l = list(Book.select(AND(*criteria)))
    l.sort()
    return l

Book.selection = staticmethod(Book_selection)

#=========================================================================
