#! /usr/bin/env python
# -*- Encoding: utf-8 -*- 

import sys
import os, os.path
import getopt
import glob

from StringIO import StringIO
from lxml import etree

from database import *

import utils

# =========================================================================

# ---- parse command line -------------

try:
    opts, args = \
        getopt.getopt(sys.argv[1:], "hv", \
                          ["help", "verbose"])
except getopt.GetoptError:
    # print help information and exit
    print >>sys.stderr, "error: usage: %s " % sys.argv[0]
    sys.exit(1)
        
# print opts
# print args
# tag = False

# =========================================================================
# Connect to database

# init(True)
init()

# =========================================================================
# Get the special Person for unvalidated books

not_on_shelf = list(Person.select(Person.q.name == 'NOT ON SHELF'))[0]

# =========================================================================
# Load the list of existing volumes

pattern = "380???????.marcxml"
notices = glob.glob(os.path.join("catalog", pattern))
notices.sort()

# =========================================================================

for notice in notices:
    vid = os.path.basename(notice).replace('.marcxml','')

    print >>sys.stderr, "Volume #%s" % vid

    # =====================================================================
    # 0. Load the Koha notice for this volume

    if not(os.path.isfile(notice)):
        print >>sys.stderr, ( "warning: something wrong with %s. skip." %
                              notice )
        # Nothing can be done for this one
        continue

    Koha_data = file(notice).read()
    Koha_data_no_xmlns = Koha_data.replace('\n  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n  xsi:schemaLocation="http://www.loc.gov/MARC21/slim http://www.loc.gov/ standards/marcxml/schema/MARC21slim.xsd"\n  xmlns="http://www.loc.gov/MARC21/slim"','')
    # data_no_xmlns = unicode_replace(data_no_xmlns)

    # Is the following a good idea ?
    # Koha_data_no_xmlns = Koha_data_no_xmlns.encode('utf-8')
    
    Koha_tree = etree.parse(StringIO(Koha_data_no_xmlns))

    authors = Koha_tree.xpath("/record/datafield[@tag='200']/subfield[@code='f']/text()")
    title = Koha_tree.xpath("/record/datafield[@tag='200']/subfield[@code='a']/text()")
    publisher = Koha_tree.xpath("/record/datafield[@tag='210']/subfield[@code='c']/text()")
    pubyear = Koha_tree.xpath("/record/datafield[@tag='210']/subfield[@code='d']/text()")

    authors = utils.unicode_fix(", ".join(authors))

    if len(title) < 1:
        print >>sys.stderr, ( "warning: %s: missing title. skip." %
                              notice )
        continue

    # print "************************"
    # print title[0]
    # print repr(title[0])
    # print utils.unicode_fix(title[0])
    # print type(title[0])
    # print dir(title[0])

    # title = title[0].decode('utf-8')
    title = utils.unicode_fix(title[0])

    if len(publisher) < 1:
        publisher = ""
    else:
        publisher = utils.unicode_fix(publisher[0])

    if len(pubyear) < 1:
        pubyear = None
    else:
        pubyear = int(pubyear[0])

    # =====================================================================
    # 1. Extract Dewey location

    Dewey_file = os.path.join("catalog", "%s.Dewey-complete" % vid)
    if not(os.path.isfile(Dewey_file)):
        print >>sys.stderr, ( "warning: %s: missing Dewey. skip." %
                              notice )
        continue
        
    l = file(Dewey_file).readlines()
    Dewey = l[0][:-1].strip()

    # =====================================================================

    barcode = vid

    # Is this book already in the database ?

    b = Book.find_by_barcode(barcode)
    if b is not None:
        continue

    # This book is a new one
 
    print "--------------------------------------------------"
    print barcode
    print title
    print authors
    # print title, type(title)
    # print authors, type(authors)
    # print publisher, type(publisher)
    # print pubyear, type(pubyear)
    # print barcode, type(barcode)
    # print Dewey, type(Dewey)

    book = Book(title = title,
                authors = authors,
                publisher = publisher,
                pubyear = pubyear,
                barcode = barcode,
                location = Dewey)

    book.person = not_on_shelf

    # That's it!

    # =====================================================================

