#! /usr/bin/env python
# -*- Encoding: utf-8 -*- 

import sys
import os, os.path
import getopt
import glob

from StringIO import StringIO
from lxml import etree

from database import *

import utils

# =========================================================================

# ---- parse command line -------------

try:
    opts, args = \
        getopt.getopt(sys.argv[1:], "hv", \
                          ["help", "verbose"])
except getopt.GetoptError:
    # print help information and exit
    print >>sys.stderr, "error: usage: %s " % sys.argv[0]
    sys.exit(1)
        
# print opts
# print args
# tag = False

# =========================================================================
# Connect to database

# init(True)
init()

# =========================================================================
# Get the special Person for unvalidated books

not_on_shelf = list(Person.select(Person.q.name == 'NOT ON SHELF'))[0]

# =========================================================================
# Load the list of existing volumes

NOTICES_PATH = "/home/llg/Desktop/LPNHE-Bibliotheque/Catalogue/LPNHE-Catalogue"
pattern = "3800??????"
notices_dir = glob.glob(os.path.join(NOTICES_PATH, pattern))
notices_dir.sort()

# =========================================================================

for notice_dir in notices_dir:

    vid = os.path.basename(notice_dir)

    print >>sys.stderr, "Volume #%s" % vid

    # =====================================================================
    # 0. Identify SUDOC marcxml file and LPNHE Dewey file
    
    ppnfile = os.path.join(notice_dir, vid + ".ppn")

    if not(os.path.isfile(ppnfile)):
        print >>sys.stderr, ( "warning: no PPN for notice %s. skip." %
                              vid )
        # Nothing can be done for this one
        continue

    ppn = file(ppnfile).read()[:-1].strip()

    # print "PPN =", ppn

    SUDOC_file = os.path.join(notice_dir, "SUDOC-" + ppn + ".marcxml")
    Dewey_file = os.path.join(notice_dir, vid + ".dewey")

    # print SUDOC_file
    # print Dewey_file

    # =====================================================================
    # 1. Load the SUDOC notice for this volume

    if not(os.path.isfile(SUDOC_file)):
        print >>sys.stderr, ( "warning: no SUDOC marcxml file for %s. skip." %
                              vid )
        # Nothing can be done for this one
        continue

    MARC_data = file(SUDOC_file).read()
    MARC_data_no_xmlns = MARC_data
    # MARC_data_no_xmlns = MARC_data.replace('\n  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\n  xsi:schemaLocation="http://www.loc.gov/MARC21/slim http://www.loc.gov/ standards/marcxml/schema/MARC21slim.xsd"\n  xmlns="http://www.loc.gov/MARC21/slim"','')
    # data_no_xmlns = unicode_replace(data_no_xmlns)

    # remove bad chars

    replacements = {""   : "", 
                    ""   : "",
                    # "cʹ"     : "c", # c cédille ? "ç" ? comment faire ?
                    "&amp;apos;" : "'",
                    "&apos;" : "'",
                    "£"      : "",
                    "¿"      : "oe"
                    }
    
    s = MARC_data_no_xmlns
    for c, v in replacements.iteritems():
        s = s.replace(c, v)
        
    MARC_data_no_xmlns = s

    # Is the following a good idea ?
    # Koha_data_no_xmlns = Koha_data_no_xmlns.encode('utf-8')
    
    MARC_tree = etree.parse(StringIO(MARC_data_no_xmlns))

    authors = MARC_tree.xpath("/record/datafield[@tag='200']/subfield[@code='f']/text()")
    title = MARC_tree.xpath("/record/datafield[@tag='200']/subfield[@code='a']/text()")
    subtitle = MARC_tree.xpath("/record/datafield[@tag='200']/subfield[@code='i']/text()")
    publisher = MARC_tree.xpath("/record/datafield[@tag='210']/subfield[@code='c']/text()")
    pubyear = MARC_tree.xpath("/record/datafield[@tag='210']/subfield[@code='d']/text()")

    authors = utils.unicode_fix(", ".join(authors))

    if len(title) < 1:
        print >>sys.stderr, ( "warning: %s: missing title. skip." %
                              vid )
        continue

    # print "************************"
    # print title[0]
    # print repr(title[0])
    # print utils.unicode_fix(title[0])
    # print type(title[0])
    # print dir(title[0])

    # title = title[0].decode('utf-8')
    title = utils.unicode_fix(title[0])
    if subtitle and (len(subtitle) >= 1):
        subtitle = utils.unicode_fix(subtitle[0])
        title += ", " + subtitle

    if len(publisher) < 1:
        publisher = ""
    else:
        publisher = utils.unicode_fix(publisher[0])

    if len(pubyear) < 1:
        pubyear = None
    else:
        pubyear = pubyear[0]
        pubyear = pubyear.replace("cop.", "")
        pubyear = pubyear.replace("Cop.", "")
        pubyear = pubyear.replace("COP.", "")
        pubyear = pubyear.replace("c", "")
        pubyear = pubyear.replace("DL", "")
        pubyear = pubyear.replace("impr.", "")
        pubyear = pubyear.replace("-", "")
        pubyear = pubyear.replace("[", "")
        pubyear = pubyear.replace("]", "")
        pubyear = pubyear.strip()
        try:
            pubyear = int(pubyear[0:4])
        except ValueError:
            pubyear = None

            

    # =====================================================================
    # 1. Extract Dewey location

    if not(os.path.isfile(Dewey_file)):
        print >>sys.stderr, ( "warning: %s: missing Dewey. skip." %
                              notice )
        continue
        
    l = file(Dewey_file).readlines()
    Dewey = l[0][:-1].strip()

    # =====================================================================

    barcode = vid

    # Is this book already in the database ?

    b = Book.find_by_barcode(barcode)
    if b is not None:
        continue

    # This book is a new one
 
    print "--------------------------------------------------"
    print barcode
    print title
    print authors
    # print title, type(title)
    # print authors, type(authors)
    # print publisher, type(publisher)
    # print pubyear, type(pubyear)
    # print barcode, type(barcode)
    # print Dewey, type(Dewey)

    book = Book(title = title,
                authors = authors,
                publisher = publisher,
                pubyear = pubyear,
                barcode = barcode,
                location = Dewey)

    book.person = not_on_shelf

    # That's it!

    # =====================================================================

